#ifndef __BV_H
#define __BV_H

#include "vector.h"
#define MAX_M 1.8447e+19 // sqrt(max_floating_value)

class Ray; // extern decl
/* this is a class that represents a 4d bounding volume */

// number of plane set normals
#define NUM_PSN 12
class BV {
public:
	static Vector normals[NUM_PSN];
	static Float normals_t[NUM_PSN]; // t components of plane set normals
	bool intersect(Ray&); // returns true if ray hits bv
	bool intersect(Ray&,Float&);// returns min t value of intersection interval
	Float dnear[NUM_PSN];
	Float dfar[NUM_PSN];
};

// a space time constraint. It basically defines an axis-alligned hyper cube
struct ST_Box{
	Float x0,x1,y0,y1,z0,z1,t0,t1;
};
extern ST_Box World_Box; // scene must lie withinn this box

// this is basically an axis-alligned box that has been tranformed
// in XYZ space (but not T space)
struct AA_Box{ // arbituarily aligned
	Vector O,X,Y,Z; // verticies 0,1,2,4
	Float t0,t1;
};

class Object;

// returns an object which is the bounding volume of the given
// object. Assumes objects bv has already been made
Object* BV_ify(Object*);

// returns false if BV is outside box
// otherwise clips the given BV to the box.
bool clip(BV&,ST_Box&);

// returns the BV that is the union of the given two
BV BV_union(BV&,BV&);


// returns true if the two volumes have any volume in common
bool intersects(BV&,AA_Box&);


#endif

