#include "camera.h"
#include "kernel.h"
#include "colour.h"
#include "output.h"
#include "linear.h"
#include <stdio.h>
#include <iostream.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>



static Camera* camera=0;
static Float time_per_frame;
static int num_frames;
static int width,height;
static Colour* scanline_buffer;
static Float* time_jitter_vals;
// our camera model is taken from Watt&Watt _AA&RT_,
// pg 8-9


static char* create_file(int frame) // creates the fname for specified frame
{
	static char buf[200];
	char num[5];
	strcpy(buf,camera->fname);
	if(buf[strlen(buf)-1]!='/') strcat(buf,"/");
	strcat(buf,"frame");
	sprintf(num,"%03d",frame);
	strcat(buf,num);

	return buf;
}

void C_render_frame(int frame)
{
	Float T=C_query_frame_time(frame);
	Ray_Info ri;
	Vector N,U,V,C;
	Vector P; // point on image plane which ray passes through
	Colour colour,sum_colour;
	Float avg_factor=1.0/Float(camera->sub*camera->sub);
	FILE* fp; // file descriptor
	char* fname;

	fname=create_file(frame);
	fp=Output_init(fname,width,height);
	if(!fp){
		cout<<"Could not open:"<<fname<<endl;
		exit(1);
	}

	for(int y=0;y<height;y++){ // for each scanline
		for(int x=0;x<width;x++){ // for each pixel in scanline
			int sub=0;
			sum_colour=Colour(0,0,0);
			for(int suby=0;suby<camera->sub;suby++){ // foreach subpixel
			for(int subx=0;subx<camera->sub;subx++){ 
				// compute origin and direction of ray;
				Float t=T;
				if(camera->motion_blur) // jitter t
				// (pxy-0.5)/sub^2 +-1/(2 sub^2)
				  t+=time_per_frame*(time_jitter_vals[sub]+
				    (K_random()/Float(camera->sub*camera->sub)-
				     1.0/Float(2*camera->sub*camera->sub) ) );
				ri.ray.t=t;
					
				C=camera->C(t);
				ri.ray.o=C;
				N=camera->look_at(t)-C;
				N.normalize();
				V=camera->up(t); // this is V prime
				V.normalize();
				V= V - (N*(V*N)); // this is V
				U=cross(N,V); // cross product
				// now work out point on image plane, P, where
				// ray passes through
				P=ri.ray.o + N*camera->f->eval(t); // center of image plane
				// now add X and Y components
				Float X,Y;
				X=x+(0.5+subx)/Float(camera->sub);
				Y=y+(0.5+suby)/Float(camera->sub);
				if(camera->anti_alias){
					X+=K_random()/Float(camera->sub)-
					   1.0/Float(2*camera->sub);
					Y+=K_random()/Float(camera->sub)-
					   1.0/Float(2*camera->sub);
				}
				P=P + V*( (height/2.0-Y)*camera->ys->eval(t)/Float(height) )
				    + U*( (X-width/2.0 )*camera->xs->eval(t)/Float(width) );
				// now ray direction is from r.o to P
				ri.ray.d=P-ri.ray.o;
				ri.ray.d.normalize();
				// now trace the ray...
				if(! K_primary_trace(ri,colour) )
					colour=camera->backdrop; // no intersection found
				sub++;
				sum_colour+=colour;
			}}
			sum_colour*=avg_factor;
			scanline_buffer[x]=sum_colour;
		}
		Output_write_scanline(fp,scanline_buffer);
		fflush(fp); // this actaully may impove how program is scheduled
	}
	Output_done(fp);
}

Float C_query_frame_time(int i)
{
	return i*time_per_frame;
}

int C_pre_query_frames(Camera* Cam)
{
	return num_frames=Cam->framesec*Cam->end_time;
}

int C_query_frames(void)
{
	return num_frames;
}

// I got these from matlabs MAGIC function
static Float timejitter1[]={0.5};
static Float timejitter2[]={
(1-0.5)/4.0 , (3-0.5)/4.0 ,
(4-0.5)/4.0 , (2-0.5)/4.0 } ;
static Float timejitter3[]={
(8-0.5)/9.0 ,(1-0.5)/9.0 ,(6-0.5)/9.0 ,
(3-0.5)/9.0 ,(5-0.5)/9.0 ,(7-0.5)/9.0 ,
(4-0.5)/9.0 ,(9-0.5)/9.0 ,(2-0.5)/9.0 } ;
static Float timejitter4[]={ // this one from intro to rt
( 7-0.5)/16.0 ,(11-0.5)/16.0 ,( 3-0.5)/16.0 ,(14-0.5)/16.0 ,
( 4-0.5)/16.0 ,(15-0.5)/16.0 ,(13-0.5)/16.0 ,( 9-0.5)/16.0 ,
(16-0.5)/16.0 ,( 1-0.5)/16.0 ,( 8-0.5)/16.0 ,(12-0.5)/16.0 ,
( 6-0.5)/16.0 ,(10-0.5)/16.0 ,( 5-0.5)/16.0 ,( 2-0.5)/16.0 };
static Float* timejittervals[]={
timejitter1,timejitter2,timejitter3,timejitter4};

void C_init(Camera *c)
{
	camera=c;
	time_per_frame=1.0/camera->framesec;
	num_frames=camera->framesec*camera->end_time;
	width=camera->width;
	height=camera->height;

	
	scanline_buffer=new Colour[width];

	time_jitter_vals=timejittervals[camera->sub-1];

	// make the directory, don't worry about any errors
	mkdir(camera->fname,S_IRWXU);
}

void C_test_results(void)
{
	cerr<<"** camera\n";
	cerr<<"Frames: "<<num_frames<<endl;
}

Camera::Camera() :
	width(200),height(200),sub(1),motion_blur(false),anti_alias(false)
{}


