#ifndef __CAMERA_H
#define __CAMERA_H

#include "config.h"
#include "vector.h"
#include "tvector.h"
#include "colour.h"
#include "bv.h"

class Source_Generator;

struct Camera{
	// these fields must be filled in
	char* fname; // name of directory frames are to be put
        int width,height; // width/height in pixels
	TVector C; //inited to NULL. default position is (0,0,0) 
	TVector look_at; // point to look at
	TVector up; // 'up' direction
	Function *f; // distance from C to image plane
	Function *xs,*ys; // scaling factor for x/y direction

	Colour backdrop; // bakdrop colour

	Float end_time; // the time at the end of the last frame
	Float framesec; // number of frames per second

	int sub; // sub by sub subpixeling
	bool motion_blur;
	bool anti_alias;

	Camera();
	void generate_code(Source_Generator*);
};

// NB: in an animation of n frames, they are numbered 0 to n-1
void C_render_frame(int i);
// called by kernel to render frame i

Float C_query_frame_time(int i);
// must return the starting time of frame i

int C_pre_query_frames(Camera*);
int C_query_frames(void);
// returns number of frames in animation

void C_init(Camera*);
// called to init the camera with the given info.

void C_test_results(void);

#endif

