#include "function.h"
#include <math.h>

static FN_constant fn_zero(0.0,0,MAX_TIME);
static FN_constant fn_unit(1.0,0,MAX_TIME);
Function* Function::zero=&fn_zero;
Function* Function::unit=&fn_unit;

#include <stdio.h>
Function::Function(Float a,Float b):
	t0(a),t1(b)
{
if(b==0.0) printf("!!!\n");}
void Function::bounds(Float& min, Float& max)
{
}

Float FN_constant::eval(Float)
{
	return c;
}

void FN_constant::bounds(Float& min, Float& max)
{
	min=c;
	max=c;
}

Float FN_linear::eval(Float t)
{
	return a*t+b;
}

void FN_linear::bounds(Float& min, Float& max)
{
	Float A=eval(t0);
	Float B=eval(t1);
	min=MIN(A,B);
	max=MAX(A,B);
}

Float FN_cos_fn::eval(Float t)
{
	return cos(fn->eval(t));
}

Float FN_sin_fn::eval(Float t)
{
	return sin(fn->eval(t));
}

Float FN_neg_sin_fn::eval(Float t)
{
	return -sin(fn->eval(t));
}


Float FN_multiply::eval(Float t){
	return a->eval(t) * b->eval(t);
}

Function* mul(Function* a,Function* b)
{
	if(a==Function::zero || b==Function::zero) return Function::zero;
	if(a==Function::unit) return b;
	if(b==Function::unit) return a;
	return new FN_multiply(a,b,MAX(a->t0,b->t0),MIN(a->t1,b->t1));
}

Float FN_add::eval(Float t){
	return f1->eval(t) + f2->eval(t);
}

Function* add(Function* a, Function *b)
{
	if(a==Function::zero) return b;
	if(b==Function::zero) return a;
	return new FN_add(a,b,MAX(a->t0,b->t0),MIN(a->t1,b->t1));
}

Function* add(Function* f1,Function* f2, Function* f3, Function* f4)
{
	Function *a,*b;
	a=add(f1,f2);
	b=add(f3,f4);
	return add(a,b);
}

Float FN_neg::eval(Float t){
	return - fn->eval(t);
}

Function* neg(Function* fn)
{
	if(fn==Function::zero) return Function::zero;
	return new FN_neg(fn,fn->t0,fn->t1);
}

Float FN_recip::eval(Float t){
	return  1.0/fn->eval(t);
}
Function* reciprical(Function*fn)
{
	if(fn==Function::unit) return Function::unit;
	return new FN_recip(fn,fn->t0,fn->t1);
}


Function* Create_FN_combine(Function* A1,Function* A2,Function* B1,
	Function* B2,Function* C1,Function* C2,Float T0,Float T1)
{
	return add(add(mul(A1,A2),mul(B1,B2)),mul(C1,C2));
}

Float FN_cos::eval(Float t)
{
	return m*cos(a*t+b)+c;
}

Float FN_sin::eval(Float t)
{
	return m*sin(a*t+b)+c;
}

Float FN_composite::eval(Float t)
{
	Function* o;
	o=fns[n-1];
	for(int i=0;i<n-1;i++)
		if(fns[i]->t0<=t && t<fns[i]->t1){
			o=fns[i];
			break;
		}
	return o->eval(t);
}

Function* Create_FN_linear_through(Float a,Float b,Float t0,Float t1)
{
	if(a==b) // make a constant function
		return new FN_constant(b,t0,t1);
	Float m=(b-a)/(t1-t0);
	return new FN_linear(m,(t0*b-t1*a)/(t0-t1),t0,t1);
}

Function* Create_FN_ease_through(Float a,Float b,Float t0, Float t1)
{
	return new FN_cos(
		PI/(t1-t0),
		PI-t0*PI/(t1-t0),
		(b-a)*0.5+a,
		(b-a)*0.5,
		t0,t1);
}


Function* Create_FN_sin(Float n, Float b, Float c, Float m,Float t0, Float t1)
{
	// a = 2*PI*(t1-t0)*n
	return new FN_sin(2*PI*n*(t1-t0),b,c,m,t0,t1);
}

Function* Create_FN_cos(Float n, Float b, Float c, Float m,Float t0, Float t1)
{
	return new FN_cos(2*PI*n*(t1-t0),b,c,m,t0,t1);
}


