#ifndef __FUNCTION_H
#define __FUNCTION_H

#include "config.h"

class Source_Generator;

// a R->R function
class Function{
public:
	Function(Float a,Float b);
	inline Float operator()(Float f){return eval(f);}
	virtual Float eval(Float)=0;
	virtual void bounds(Float& min, Float& max); // default uses numerical?
	static Function* zero; // always 0
	static Function* unit; // always 1

	Float t0,t1; // domain
};

// function operators

Function* mul(Function*,Function*);
Function* add(Function*,Function*,Function*,Function* =Function::zero);
Function* add(Function*,Function*);
Function* neg(Function*);
Function* reciprical(Function*);

class FN_constant: public Function{
private:
	Float c;
public:
	FN_constant(Float C,Float a,Float b):
		Function(a,b), c(C){}
	virtual Float eval(Float);
	virtual void bounds(Float&, Float&);
};

class FN_linear: public Function{
private:
	Float a,b; // at+b
public:
	FN_linear(Float A, Float B, Float T0,Float T1):
		Function(T0,T1),a(A),b(B){}
	virtual Float eval(Float);
	virtual void bounds(Float&, Float&);
};

class FN_cos_fn: public Function{
private:
	Function *fn;
public:
	FN_cos_fn(Function *f,Float T0,Float T1):
		Function(T0,T1),fn(f){}
	virtual Float eval(Float);
};

class FN_sin_fn: public Function{
private:
	Function *fn;
public:
	FN_sin_fn(Function *f,Float T0,Float T1):
		Function(T0,T1),fn(f){}
	virtual Float eval(Float);
};


class FN_neg_sin_fn: public Function{
private:
	Function *fn;
public:
	FN_neg_sin_fn(Function *f,Float T0,Float T1):
		Function(T0,T1),fn(f){}
	virtual Float eval(Float);
};

// m*cos(a*t + b ) + c
class FN_cos: public Function{
private:
	Float a,b,c,m;
public:
	FN_cos(Float A, Float B, Float C,Float M,Float T0, Float T1):
		Function(T0,T1),a(A),b(B),c(C),m(M){};
	virtual Float eval(Float);
};

// m*sin(a*t + b ) + c
class FN_sin: public Function{
private:
	Float a,b,c,m;
public:
	FN_sin(Float A, Float B, Float C, Float M,Float T0, Float T1):
		Function(T0,T1),a(A),b(B),c(C),m(M){};
	virtual Float eval(Float);
};

// a composite function
class FN_composite: public Function{
private:
	int n;
	Function** fns;
public:
	FN_composite(int N, Function** Fns,Float T0, Float T1):
		Function(T0,T1),n(N),fns(Fns){}
	virtual Float eval(Float);
};

class FN_multiply:public Function{
private:
	Function *a,*b;
public:
	FN_multiply(Function *A, Function *B,Float T0,Float T1):
		Function(T0,T1),a(A),b(B){}
	virtual Float eval(Float);
};

class FN_add: public Function{
private:
	Function *f1,*f2;
public:
	FN_add(Function *A, Function *B,Float T0,Float T1):
		Function(T0,T1),f1(A),f2(B){}
	virtual Float eval(Float);
};

class FN_neg: public Function{
private:
	Function *fn;
public:
	FN_neg(Function *f,Float T0,Float T1):
		Function(T0,T1),fn(f){}
	virtual Float eval(Float);
};

class FN_recip: public Function{
private:
	Function *fn;
public:
	FN_recip(Function *f,Float T0,Float T1):
		Function(T0,T1),fn(f){}
	virtual Float eval(Float);
};


// this returns a fn that is a line that goes from a to b, starting at t0,
// ending at t1
Function* Create_FN_linear_through(Float a,Float b,Float t0,Float t1);
Function* Create_FN_ease_through(Float a,Float b,Float t0, Float t1);

// these functions create sin/cos that exit over t0 to t1, and have
// n periods over that range, plus with the given b,c and m components
Function* Create_FN_sin(Float n, Float b, Float c, Float m,Float t0, Float t1);
Function* Create_FN_cos(Float n, Float b, Float c, Float m,Float t0, Float t1);

// returns a function that  is a1*a2 + b1*b2 + c1*c2
Function* Create_FN_combine(Function* A1,Function* A2,Function* B1,
	Function* B2,Function* C1,Function* C2,Float T0,Float T1);

#endif


