#include "fn_math.h"

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <signal.h>

static FILE *Input,*Output;
static pid_t PID;

void M_init()
{
	// establish link to matlab
	int input[2];
	int output[2];
	pipe(input);
	pipe(output);
	if((PID=fork())==0){ // child
		// setup childs input and output
		close(output[0]); dup2(output[1],0);
		close(input[0]);  dup2(input[1],1);
		close(output[1]); close(input[1]);
		char* args[]={"","-display","none:0",0};
		execvp("matlab",args);
		perror("Could not start matlab");
		exit(-1);
	}
	// parent
	close(output[1]); close(input[1]);
	Input=fdopen(input[0],"r");
	Output=fdopen(output[0],"w");
	// wait till we get the >> prompt
	int prompt_count=0;
	while(prompt_count!=2){
		int c=fgetc(Input);
		if(c==EOF){
			fprintf(stderr,"Error using matlab, killing....\n");
			kill(PID,SIGKILL);
			exit(-1);
		}
		if(c=='>') prompt_count++;
		else prompt_count=0;
	}
	printf("Matlab opened.\n");
	// start my matlab interface script
	fprintf(Output,"matlabif\n");
	fflush(Output);
	// get handshake
	prompt_count=0;
	while(prompt_count!=2){
		int c=fgetc(Input);
		if(c==EOF){
			fprintf(stderr,"Error with matlab interface. killing....\n");
			kill(PID,SIGKILL);
			exit(-1);
		}
		if(c=='^') prompt_count++;
		else prompt_count=0;
	}
}

void M_shutdown()
{
	fprintf(Output,"#\n");
	fclose(Output);
	fclose(Input);
	sleep(1);
	kill(PID,SIGKILL);
}

void M_bounds(char *s,Float&min,Float&max)
{
	fprintf(Output,"%s\n",s);
	fflush(Output);
	double a,b;
	fscanf(Input,"%lf%lf",&a,&b);
	min=a; max=b;
}
