#include <stdio.h>
#include <iostream.h>
#include <ctype.h>
#include "nff.h"
#include "polygon.h"
#include "spheres.h"

/*static*/ struct NFF_Field{
	enum Type {none,viewpoint,background,light,fill,cone,sphere,
	  polygon,polypatch} type;
	union{
		struct{
			int nverts;
		} poly;
	};
	Surface_Info si;
	Vector V1;
	Float f1;
};

static Float t0,t1;
static char buf[256];
// these functions more or less assume that there are no comments
// inside a field
static NFF_Field NFF_read_head(FILE* fp)
{
	NFF_Field f;
	f.type=NFF_Field::none;
	char* s;
	buf[0]=0;
	s=fgets(buf,256,fp);
	while(s&&(buf[0]=='#'||buf[0]==0||buf[0]=='\n')){ // skip comments / blank lines
		s=fgets(buf,256,fp);
	}
	if(!s){ // EOF or error
		f.type=NFF_Field::none;
		return f;
	}
	double f1,f2,f3,f4,f5,f6,f7,f8;
	switch(buf[0]){
		case 'v': f.type=NFF_Field::viewpoint ;
			break;
		case 'b': f.type=NFF_Field::background ;
			break;
		case 'l': f.type=NFF_Field::light ;
			break;
		case 'f': f.type=NFF_Field::fill ;
			sscanf(buf+1,"%lf%lf%lf%lf%lf%lf%lf%lf",&f1,&f2,&f3,&f4,&f5,&f6,&f7,&f8);
			f.si.tcolour=f.si.rcolour=f.si.dcolour=Colour(f1,f2,f3);
			f.si.kdr=f4;
			f.si.ksr=f5;
			f.si.n=f6;
			if(f6==0.0) f.si.n=1;
			f.si.kst=f7;
			if(f7>0) f.si.ior=f8;
			else f.si.ior=0;
			break;
		case 'c': f.type=NFF_Field::cone ;
			break;
		case 's': f.type=NFF_Field::sphere;
			sscanf(buf+1,"%lf%lf%lf%lf",&f1,&f2,&f3,&f4);
			f.V1=Vector(f1,f2,f3);
			f.f1=f4;
			break;
		case 'p':
			if(buf[1]=='p') f.type=NFF_Field::polypatch;
			else f.type=NFF_Field::polygon;
			sscanf(buf+2,"%d",&(f.poly.nverts));
			break;
	}
	return f;
}

static void NFF_read_skip(FILE* fp,NFF_Field field)
{
	int i;
	float f;
	switch(field.type){
		case NFF_Field::none: break;
		case NFF_Field::background :
		case NFF_Field::light :
		case NFF_Field::fill :
		case NFF_Field::cone :
		case NFF_Field::sphere :
			// these just are one liners anyway
			break;
		case NFF_Field::viewpoint:
			cout<<"NFF: skip not implemented yet"<<endl;
			break;
		case NFF_Field::polygon :
			for(i=0;i<field.poly.nverts;i++) fscanf(fp,"%f%f%f",&f,&f,&f);
			break;
		case NFF_Field::polypatch :
			for(i=0;i<field.poly.nverts;i++) fscanf(fp,"%f%f%f%f%f%f",&f,&f,&f,&f,&f,&f);
			break;
	}
}

#define MAXVERTS 50
static Object* NFF_read_obj(FILE* fp, NFF_Field field)
{
	static Vector verts[MAXVERTS];
	static Vector norms[MAXVERTS];
	static Surface_Info last_si=Surface_Info_default;
	double f1,f2,f3;
	int i;
	switch(field.type){
		case NFF_Field::none: break;
		case NFF_Field::viewpoint:
		case NFF_Field::background :
		case NFF_Field::light :
		case NFF_Field::cone :
			cout<<"NFF: no readed implemented"<<endl;
			NFF_read_skip(fp,field);
			return 0;
			break;
		case NFF_Field::sphere :
			return Create_Static_Sphere(field.V1,field.f1,t0,t1,last_si);
			break;
		case NFF_Field::fill :
			last_si=field.si;
			return 0;
			break;
		case NFF_Field::polygon :
			for(i=0;i<field.poly.nverts;i++){
				fscanf(fp,"%lf%lf%lf",&f1,&f2,&f3);
				verts[i]=Vector(f1,f2,f3);
			}
			return Create_Polygon(field.poly.nverts,verts,t0,t1,last_si);
			break;
		case NFF_Field::polypatch :
			for(i=0;i<field.poly.nverts;i++){
				fscanf(fp,"%lf%lf%lf",&f1,&f2,&f3);
				verts[i]=Vector(f1,f2,f3);
				fscanf(fp,"%lf%lf%lf",&f1,&f2,&f3);
				norms[i]=Vector(f1,f2,f3);
			}
			return Create_Polygon(field.poly.nverts,verts,t0,t1,last_si);
			break;
	}
	return 0;
}

Object** NFF_read_file(char* fname, int& nobjects,Float T0,Float T1)
{
	t0=T0; t1=T1;
	FILE *fp;
	// scan through the file first and count how many objects there
	// are

	fp=fopen(fname,"r");
	if(!fp){
		cout<<"Could not open '"<<fname<<"'"<<endl;
		return 0;
	}
	nobjects=0;
	NFF_Field field;
	do{
		field=NFF_read_head(fp);
		if(field.type== NFF_Field::polygon ||
		  field.type== NFF_Field::polypatch ||
		  field.type== NFF_Field::sphere) nobjects++;
		NFF_read_skip(fp,field);
	} while (field.type!=NFF_Field::none);
	fclose(fp);
	// now read them in;
	
	fp=fopen(fname,"r");
	if(!fp){
		cout<<"Could not open '"<<fname<<"'"<<endl;
		return 0;
	}
	Object** objs=new Object*[nobjects];
	int n=0;
	while(n<nobjects){
		field=NFF_read_head(fp);
		if(field.type== NFF_Field::polygon ||
		  field.type== NFF_Field::polypatch ||
		  field.type== NFF_Field::sphere){
			objs[n++]=NFF_read_obj(fp,field);
		}
		else if(field.type==NFF_Field::fill) NFF_read_obj(fp,field);
	}
	fclose(fp);
	return objs;
}

