#ifndef __OBJECT_H
#define __OBJECT_H

#include "kernel.h"
#include "colour.h"

struct L_proj_fn;
class Source_Generator;

struct Surface_Info{
	Surface_Info();
	Float kdr,ksr,kst;
	Float n; // phong coefficient
	Colour rcolour,tcolour,dcolour;
	Float ior; // index of refraction, 0 for opaque
};
extern Surface_Info Surface_Info_default;


// used by shader to get normal info, etc
struct Shade_Info{
	Vector normal;
	Vector P; //point of intersection
};
// the abstract class the encapsulates all the
// functions an object primitive must provide
class Object {
public:
	enum Type {primitive,transform,ttransform};
	Object(Surface_Info,Float T0,Float T1,Type);
	// interface members

	virtual bool intersect(Ray&,Float&)=0;
	// called to see if ray intersects this object. if it does,
	// then the appropriate value of t is also returned

	virtual void aux_info(Ray&,Intersect_Info&,Shade_Info&)=0;
	// passes the ray_info and i_info of intersection, so as to
	// fill in shade_info. Normal needs to be returned unit length

	virtual void compute_bounds(BV&);
	// computes a bounding volume. default provided for primitives.

	virtual void compute_extent(L_proj_fn&,Float &min,Float& max,Float T0,Float T1){}
	// for primitives only. compute (min,max) of f() constrained
	// to g(x) (for this primitive type) 

	virtual bool inside(AA_Box&)=0;
	// returns true if any of the *suface* of this object lies within
	// the aabox
	// false otherwise

	Surface_Info surface;
	Object* next;
	Type type;
	Float t0,t1;
	BV bound;
	typedef unsigned long MailBox;
	MailBox mailbox;
	Float mailbox_t;
};

#endif

