
#include "objectmanager.h"
#include "object.h"
#include "light.h"
#include <iostream.h>

/* an implementation of a naive (exaustive) object manager */

static Object* obj_list;
static Light* light_list;


// testing paramaters
static Counter rayobject_tests=0; // number of ray/object intersections
static Counter rayobject_hits=0; // number of tests that hit
static Counter raybv_tests=0;
void OM_init(Object* o, Light *l)
{
	obj_list=o;
	light_list=l;
}

bool OM_intersect(Ray_Info& ri ,Intersect_Info& ii)
{
	Object* ptr=obj_list;
	bool found=false;
	Float best_t=0.0,t;

	while(ptr){
		raybv_tests++;
		if(ptr->bound.intersect(ri.ray)){
		rayobject_tests++;
		if(ptr->intersect(ri.ray,t)){
			rayobject_hits++;
			// ok it intersected, is it better than what we got?
			if(!found || t<best_t){ // got new closest intersection
				found=true;
				best_t=t;
				ii.t=t;
				ii.obj=ptr;
			}
		}
		}
		ptr=ptr->next;
	}
	return found;
}


bool OM_shadow_test(Ray& ray, Float d,Light* L)
{
	Float t;
	// test the cache
	if(L->last_blocked){
		if(L->last_blocked->intersect(ray,t) && t<d) return true;
	}

	Object *ptr=obj_list;
	while(ptr){
		raybv_tests++;
		if(ptr->bound.intersect(ray)){
		rayobject_tests++;
		if(ptr->intersect(ray,t)){
			rayobject_hits++;
			if(t<d){
				L->last_blocked=ptr;
				return true;
			}
		}}
		ptr=ptr->next;
	}
	return false;
}

void OM_starting_frame(int)
{
	// doopy doo
}

void OM_test_results(void)
{
	cerr<<"** Object Manager: Exhaustive\n";
	cerr<<"Ray/BV tests     :"<<raybv_tests<<endl;
	cerr<<"Ray/Object tests :"<<rayobject_tests<<endl;
	cerr<<"Ray/Object hits  :"<<rayobject_hits<<endl;
}
