#include "kernel.h"
#include "scene.h"
#include "camera.h"
#include "object.h"
#include "light.h"
#include "sourcegen.h"
#include "fn_math.h"
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>

static Kernel_Info kernel_info;

/* phase 1 kernel */


// lists of objects and lights
static Object* object_list=0;
static Light* light_list=0;
// the camera
static Camera* cam;


void K_main( void)
{
	cout<<"Phase 1: compile."<<endl;

	// create the scene

	// kernel defaults
	kernel_info.min_weight=0.05;
	kernel_info.max_depth=10;
	kernel_info.ambient=Colour(0.3,0.3,0.3);
	kernel_info.bounding_colour=Colour(0,0,0);

	user_create_scene(kernel_info);

	// for each object, compute it's bounding volume
	Object* o_ptr=object_list;
	while(o_ptr){
		o_ptr->compute_bounds();
		o_ptr=o_ptr->next;
	}

	// create the .cc file (phew)
	Source_Generator sg("SCENE.cc");
	// generate code for objects
	o_ptr=object_list;
	while(o_ptr){
		o_ptr->generate_code(&sg,true);
		o_ptr=o_ptr->next;
	}
	// generate code for lights
	Light* l_ptr=light_list;
	while(l_ptr){
		Light_generate_code(l_ptr,&sg);
		l_ptr=l_ptr->next;
	}
	// generate code for camera
	cam->generate_code(&sg);

	sg.generate_code(&kernel_info);

}

void K_add_object(Object* obj)
{
	obj->next=object_list;
	object_list=obj;
}

void K_add_light(Light* l)
{
	l->next=light_list;
	light_list=l;
}

void K_init_camera(Camera* Cam)
{
	cam=Cam;
}


Float K_random()
{
	return (rand()&0xffff)*(1.0/0xffff);
}


// just stubs
bool K_trace(Ray_Info&, Colour&){return false;}
bool K_primary_trace(Ray_Info&, Colour&){return false;}
bool K_shadow_test(Ray&,Float,Light*){return false;}
Light* K_get_lights(){return 0;}


