#include "kernel.h"
#include "scene.h"
#include "objectmanager.h"
#include "camera.h"
#include "object.h"
#include "light.h"
#include "shader.h"
#include "timer.h"
#include "linear.h"
#include <iostream.h>
#include <stdlib.h>

static Kernel_Info kernel_info;

// kernel.cc
// variables used to store testing information
static Counter primary_rays, transmitted_rays, reflected_rays, shadow_rays;
static Counter num_objects=0;
static Timer creating_time, om_init_time, rendering_time,computebv_time;

// lists of objects and lights
static Object* object_list=0;
static Light* light_list=0;

static void K_test_results();

static Camera cam;
void K_main( void)
{
	cout<<"Phase 2: render."<<endl;
	// init testing vars
	primary_rays=transmitted_rays=reflected_rays=shadow_rays=0;

	// create the scene
        // kernel defaults
	kernel_info.min_weight=0.05;
	kernel_info.max_depth=10;
	kernel_info.ambient=Colour(0.3,0.3,0.3);
	kernel_info.bounding_colour=Colour(0,0,0);
	kernel_info.seg_sub=1;
	kernel_info.bv_ify=false;
	creating_time.start();
	user_create_scene(kernel_info,cam);
	creating_time.stop();

	// create bounding volumes
	cout<<"Creating Bounding Volumes"<<endl;
	computebv_time.start();
	L_init(C_pre_query_frames(&cam)*kernel_info.seg_sub );
	C_init(&cam);
	Object* o_ptr=object_list;
	while(o_ptr){
		o_ptr->compute_bounds(o_ptr->bound);
		o_ptr=o_ptr->next;
	}
	computebv_time.stop();
	// bv_ify if specified
	if(kernel_info.bv_ify){
		Object* new_head;
		new_head=BV_ify(object_list);
		o_ptr=object_list->next;
		while(o_ptr){ // this currently reverses order of object list
			Object*temp = BV_ify(o_ptr);
			temp->next=new_head;
			new_head=temp;
			o_ptr=o_ptr->next;
		}
		object_list=new_head;
	}


	cout<<"Object Mananger Init."<<endl;
	S_init(kernel_info.ambient,kernel_info.bounding_colour);
	om_init_time.start();
	OM_init(object_list,light_list);
	om_init_time.stop();

	int num_frames=C_query_frames();
	int frame;

	cout<<"Rendering"<<endl;
	rendering_time.start();
	for(frame=0;frame<num_frames;frame++){
		OM_starting_frame(frame);
		C_render_frame(frame);
	}
	rendering_time.stop();

	// output statistics
	K_test_results();
}

bool K_trace(Ray_Info& ri, Colour& colour)
{
	if(ri.weight<kernel_info.min_weight || ri.generation>kernel_info.max_depth)
		return false;

	if(ri.type==Ray_Info::transmitted) transmitted_rays++;
	else if(ri.type==::Ray_Info::reflected)  reflected_rays++;
	else primary_rays++;

	Intersect_Info ii;
	if(OM_intersect(ri,ii)){
		S_shade(ri,ii,colour);
		return true;
	}
	else
		return false;
}

bool K_primary_trace(Ray_Info& ri, Colour& colour)
{
	ri.weight=1.0;
	ri.generation=0;
	ri.type=Ray_Info::primary;
	ri.object_in=0;
	ri.ray.bv_setup();

	colour=Colour(0,0,0);

	return K_trace(ri,colour);
}

bool K_shadow_test(Ray& r,Float d,Light* l)
{
	shadow_rays++;
	return OM_shadow_test(r,d,l);
}

void K_add_object(Object* obj)
{
	obj->next=object_list;
	object_list=obj;
	num_objects++;
}

void K_add_light(Light* l)
{
	l->next=light_list;
	light_list=l;
}


Light* K_get_lights()
{
	return light_list;
}

Float K_random()
{
	return drand48();
}

static void K_test_results()
{
	cerr<<"** kernel\n";
	cerr<<"p/r/t/s rays     :"<<primary_rays<<" "<<reflected_rays<<" "<<
		transmitted_rays<<" "<<shadow_rays<<endl;
	cerr<<"Rendering time   :"<<rendering_time<<endl;
	cerr<<"Scene build time :"<<creating_time<<endl;
	cerr<<"Compute BV time  :"<<computebv_time<<endl;
	cerr<<"OM init time     :"<<om_init_time<<endl;
	cerr<<"Number of Objects:"<<num_objects<<endl;
	OM_test_results();
	C_test_results();
}

#include "phase_2.h"

Float FN_C::eval(Float t)
{
        return (*fn)(t);
}

void FN_C::bounds(Float&,Float&)
{}// should never be called

char* FN_C::C_string_rep(Source_Generator*)
{return 0;}// shoule never be called



