#include "kernel.h"
#include "linear.h"
#include "spheres.h"
#include "polygon.h"
#include "ttransform.h"
#include "transform.h"


bool intersects(BV& bv,AA_Box& box)
{
	// check if times overlap
	if(bv.dnear[3]>box.t1 || bv.dfar[3]<box.t0) return false;
}


bool StaticSphere::inside(AA_Box& box)
{
	if(box.t1<t0 || box.t0>t1) return false;
	// taken from graphics gems I

	// the coordiante-axis for the box
	Vector X,Y,Z;
	X=box.X-box.O;
	Y=box.Y-box.O;
	Z=box.Z-box.O;
	//X.normalize();
	//Y.normalize();
	//Z.normalize();
	Float dmax=0, dmin=0;
	Float a,b,Ci;
	Float Bmin,Bmax;
	
	// loop is unrolled
	// X
	Ci=X*p;
	a=X*box.O; b=X*box.X;
	if(a<b) { Bmin=a; Bmax=b;}
	else { Bmin=b; Bmax=a;}
	a=Ci-Bmin; a*=a;
	b=Ci-Bmax; b*=b;
	dmax+=MAX(a,b);
	if(! (Bmin<=Ci && Ci<=Bmax) ) dmin+=MIN(a,b);
	// Y
	Ci=Y*p;
	a=Y*box.O; b=Y*box.Y;
	if(a<b) { Bmin=a; Bmax=b;}
	else { Bmin=b; Bmax=a;}
	a=Ci-Bmin; a*=a;
	b=Ci-Bmax; b*=b;
	dmax+=MAX(a,b);
	if(! (Bmin<=Ci && Ci<=Bmax) ) dmin+=MIN(a,b);
	// Z
	Ci=Z*p;
	a=Z*box.O; b=Z*box.Z;
	if(a<b) { Bmin=a; Bmax=b;}
	else { Bmin=b; Bmax=a;}
	a=Ci-Bmin; a*=a;
	b=Ci-Bmax; b*=b;
	dmax+=MAX(a,b);
	if(! (Bmin<=Ci && Ci<=Bmax) ) dmin+=MIN(a,b);

	if( dmin<=r2 && r2<=dmax) return true;
	return false;
}


bool Transformed_Object::inside(AA_Box& Box)
{
	if(Box.t1<t0 || Box.t0>t1) return false;
	AA_Box box;
	// transform Box to local co-ords
	box.O=transform_point(T.inverse,Box.O);
	box.X=transform_point(T.inverse,Box.X);
	box.Y=transform_point(T.inverse,Box.Y);
	box.Z=transform_point(T.inverse,Box.Z);
	box.t0=Box.t0; box.t1=Box.t1;
	return obj->inside(box);
}

bool TTransformed_Object::inside(AA_Box& Box)
{
	if(Box.t1<t0 || Box.t0>t1) return false;
	Float T0=Box.t0, T1=Box.t1;
	if(T0>T->t0) T0=T->t0;
	if(T1<T->t1) T1=T->t1;

	int first_index, last_index;
	first_index=floor(max_segments*(T0-t0)/(t1-t0));
	last_index=floor(max_segments*(T1-t0)/(t1-t0));
	double segment_width=(T->t1-T->t0)/double(max_segments);

	Float last_t=T0-1; AA_Box last_box;

	AA_Box box;
	for(int index=first_index; index<=last_index; index++){
		box.t0=MAX(T0,segment_width*index+T->t0);
		box.t1=MIN(T1,segment_width*(index+1)+T->t0);

		if(box.t1!=last_t){
			// transform Box to local co-ords
			T->eval_inverse(box.t1);
			box.O=transform_point(T->inverse,Box.O);
			box.X=transform_point(T->inverse,Box.X);
			box.Y=transform_point(T->inverse,Box.Y);
			box.Z=transform_point(T->inverse,Box.Z);
		} else box=last_box;
		if(obj->inside(box)) return true;

		// transform Box to local co-ords
		T->eval_inverse(box.t0);
		box.O=transform_point(T->inverse,Box.O);
		box.X=transform_point(T->inverse,Box.X);
		box.Y=transform_point(T->inverse,Box.Y);
		box.Z=transform_point(T->inverse,Box.Z);
		if(obj->inside(box)) return true;
		last_t=box.t0;
		last_box=box;
	}
	return false;
}

// some GraphicsGems3 code
#include "triangleCube.cc"

/* this function takes an AA_Box and returns a matrix
   that will transform points in world co-ordinates to
   normal-box co-ordinates */
static void change_basis(AA_Box& box, Matrix &M)
{ // from pg 220 of Foley
	Vector Px=0.5*(box.X-box.O);
	Vector Py=0.5*(box.Y-box.O);
	Vector Pz=0.5*(box.Z-box.O);
	Vector Rx=(1.0/(Px*Px))*Px;
	Vector Ry=(1.0/(Py*Py))*Py;
	Vector Rz=(1.0/(Pz*Pz))*Pz;

	M.r1=Rx.x; M.r2=Rx.y; M.r3=Rx.z;
	M.r4=Ry.x; M.r5=Ry.y; M.r6=Ry.z;
	M.r7=Rz.x; M.r8=Rz.y; M.r9=Rz.z;
	Vector C=box.O+Px+Py+Pz;
	M.tx=-C.x; M.ty=-C.y; M.tz=-C.z;
}
/* 
  The t_c_intersection(Triangle3& t) in the above file
  checks the triangle against a unit cube centered at the
  origin. So we have to transform the verticies of the
  polygon *to* the normalize local co-ordinate system of
  the box */
bool Polygon::inside(AA_Box& Box)
{
	Matrix M;
	change_basis(Box,M);
	Triangle3 T;
	T.v1=transform_point(M,verts[0]);
	T.v2=transform_point(M,verts[1]);
	T.v3=transform_point(M,verts[2]);
	if(t_c_intersection(T)==INSIDE) return true;
	for(int i=3;i<nverts;i++){
		T.v2=T.v3;
		T.v3=transform_point(M,verts[i]);
		if(t_c_intersection(T)==INSIDE) return true;
	}
	return false;
}

