#include "sourcegen.h"

#include "object.h"
#include "ttransform.h"
#include "light.h"
#include "camera.h"
#include "scene.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

Source_Generator::Source_Generator(char*Fname):
	fname(Fname),camera_code(0)
{
}

void Source_Generator::generate_code(Kernel_Info* ki)
{
	FILE* fp=fopen(fname,"w");
	if(!fp){
		printf("Could not open %s.",fname);
		exit(1);
	}
	// include files;
	fprintf(fp,"#include \"phase_2.h\"\n\n");

	List<Function*>::Node *fn_ptr=functions.head;
	List<Object*>::Node *obj_ptr=objects.head;
	List<Light*>::Node *light_ptr=lights.head;
	List<Transformation*>::Node *trans_ptr=trans.head;
	List<TTransformation*>::Node *ttrans_ptr=ttrans.head;
	List<char*>::Node *str_ptr=function_code.head;
	char id_buf[SG_ID_LEN];

	// all the functions

	// declare the C functions and Functions
	while(fn_ptr){
		fprintf(fp,"Float C_%s(Float);\n",name_of(fn_ptr->data,id_buf));
		fprintf(fp,
			"Function* %s=new FN_C(C_%s,%.14lg,%.14lg);\n",
			id_buf,id_buf,double(fn_ptr->data->t0),double(fn_ptr->data->t1));
		fn_ptr=fn_ptr->next;
	}

	// define the C functions
	fn_ptr=functions.head;
	while(fn_ptr){
		fprintf(fp,"Float C_%s(Float t){\n",name_of(fn_ptr->data,id_buf));
		fprintf(fp,"%s\n}\n",str_ptr->data);
		fn_ptr=fn_ptr->next;
		str_ptr=str_ptr->next;
	}

	// declare camera
	fprintf(fp,"static Camera cam;\n");
	// start create function
	fprintf(fp,"void user_create_scene(Kernel_Info& ki){\n");

	// declare all the objects
	while(obj_ptr){
		fprintf(fp,"Object* %s;\n",name_of(obj_ptr->data,id_buf));
		obj_ptr=obj_ptr->next;
	}
	
	// declare all the transformations
	// define all the transformations
	str_ptr=trans_code.head;
	while(trans_ptr){
		fprintf(fp,"Transformation* %s;\n",name_of(trans_ptr->data,id_buf));
		fprintf(fp,str_ptr->data);
		trans_ptr=trans_ptr->next;
		str_ptr=str_ptr->next;
	}
	str_ptr=ttrans_code.head;
	while(ttrans_ptr){
		fprintf(fp,"TTransformation* %s;\n",name_of(ttrans_ptr->data,id_buf));
		fprintf(fp,str_ptr->data);
		ttrans_ptr=ttrans_ptr->next;
		str_ptr=str_ptr->next;
	}
	// define all the objects
	// do the primitives first, then the others
	obj_ptr=objects.head;
	str_ptr=object_code.head;
	while(obj_ptr){
		if(obj_ptr->data->type==Object::primitive){
			fprintf(fp,str_ptr->data);
			if(obj_ptr->data->root)
				fprintf(fp,"K_add_object(%s);\n",name_of(obj_ptr->data,id_buf));
		}
		obj_ptr=obj_ptr->next;
		str_ptr=str_ptr->next;
	}
	// non primitives
	obj_ptr=objects.head;
	str_ptr=object_code.head;
	while(obj_ptr){
		if(obj_ptr->data->type!=Object::primitive){
			fprintf(fp,str_ptr->data);
			if(obj_ptr->data->root)
				fprintf(fp,"K_add_object(%s);\n",name_of(obj_ptr->data,id_buf));
		}
		obj_ptr=obj_ptr->next;
		str_ptr=str_ptr->next;
	}
	// all the lights
	str_ptr=light_code.head;
	while(light_ptr){
		fprintf(fp,str_ptr->data);
		light_ptr=light_ptr->next;;
		str_ptr=str_ptr->next;
	}

	// the camera;
	if(camera_code)fprintf(fp,camera_code);
	fprintf(fp,"K_init_camera(&cam);\n");
	// the kernel info
	fprintf(fp,"ki.min_weight=%.14lg;\n",ki->min_weight);
	fprintf(fp,"ki.max_depth=%.14lg;\n",ki->max_depth);
	fprintf(fp,"ki.ambient=Colour(%.14lg,%.14lg,%.14lg);\n",
		ki->ambient.r,ki->ambient.g,ki->ambient.b);
	fprintf(fp,"ki.bounding_colour=Colour(%.14lg,%.14lg,%.14lg);\n",
		ki->bounding_colour.r,ki->bounding_colour.g,ki->bounding_colour.b);

	fprintf(fp,"}\n");
	fclose(fp);
}

void Source_Generator::add_object_code(Object* p,char* c)
{
	if(objects.is_in(p))return;
	objects.add(p);
	object_code.add(STR_dup(c));
}

void Source_Generator::add_transform_code(Transformation* p,char* c)
{
	if(trans.is_in(p))return;
	trans.add(p);
	trans_code.add(STR_dup(c));
}

void Source_Generator::add_ttransform_code(TTransformation* p,char* c)
{
	if(ttrans.is_in(p))return;
	ttrans.add(p);
	ttrans_code.add(STR_dup(c));
}

void Source_Generator::add_function_code(Function* p,char* c)
{
	if(functions.is_in(p))return;
	functions.add(p);
	function_code.add(STR_dup(c));
}

void Source_Generator::add_light_code(Light* p,char* c)
{
	if(lights.is_in(p))return;
	lights.add(p);
	light_code.add(STR_dup(c));
}

void Source_Generator::add_camera_code(Camera*,char* c)
{
	camera_code=STR_dup(c);
}


char* Source_Generator::name_of(Object* p,char* buf)
{
	sprintf(buf,"OB_%p",p);
	return buf;
}

char* Source_Generator::name_of(Transformation* p,char* buf)
{
	sprintf(buf,"TR_%p",p);
	return buf;
}

char* Source_Generator::name_of(TTransformation* p,char* buf)
{
	sprintf(buf,"TT_%p",p);
	return buf;
}

char* Source_Generator::name_of(Function* p,char* buf)
{
	sprintf(buf,"FN_%p",p);
	return buf;
}

char* Source_Generator::name_of(Light* p,char* buf)
{
	sprintf(buf,"LI_%p",p);
	return buf;
}

char* STR_dup(char* s)
{
	char* r=new char[strlen(s)+1];
	strcpy(r,s);
	return r;
}

char* STR_add(char* a,char* b)
{
	char *s=new char[strlen(a)+strlen(b)+1];
	strcpy(s,a);
	strcat(s,b);
	return s;
}

char *STR_add(int n, char**S)
{
	int i;
	int len=0;
	for(i=0;i<n;i++) len+=strlen(S[i]);
	char *s;
	s=new char[len+1]; *s=0;
	for(i=0;i<n;i++) strcat(s,S[i]);
	return s;
}


