#include "output.h"
#include <string.h>

/*
  implements an output stream as a TARGA file
  TARGA file info taken from the "GRAPHIC FILE FORMATS HOME PAGE"
*/

static int width,height;

inline static void write_byte(FILE* fp, int b)
{
	char c=b&0xff;
	fwrite(&c,1,1,fp);
}

inline static void write_word(FILE* fp, int b)
{
	// bytes written in lo-hi order
	char c=b&0xff;
	fwrite(&c,1,1,fp);
	c=(b>>8)&0xff;
	fwrite(&c,1,1,fp);
}

inline static int clean(Float f){
	int a=f*255.0;
	if(a<0) return 0;
	return a>255?255:a;
}

inline static void write_colour(FILE* fp,Colour& c)
{
	write_byte(fp,clean(c.b));
	write_byte(fp,clean(c.g));
	write_byte(fp,clean(c.r));
}

FILE* Output_init(char* fname,int w, int h)
{
	FILE* fp;

	strcat(fname,".tga");
	fp=fopen(fname,"wb");
	if(!fp) return 0;

	width=w; height =h;
	write_byte(fp,0);    // no ID field
	write_byte(fp,0);    // no colour map included
	write_byte(fp,2);    // an unmapped RGB image
	write_byte(fp,0);    // ignorred colourmap specification
	write_word(fp,0); write_word(fp,0);
	write_word(fp,0);    // x origin
	write_word(fp,0);    // y origin
	write_word(fp,width);// width
	write_word(fp,height);// height
	write_byte(fp,24);   // 24 bit image
	write_byte(fp,0x21); // 0010001 Image descriptor byte (non-i, up-l, tga24)
	// header compete, image data goes after this
	return fp;
}

void Output_write_scanline(FILE* fp,Colour* scanline)
{
	int i;
	for(i=0;i<width;i++){
		write_colour(fp,scanline[i]);
	}
}

void Output_done(FILE* fp)
{
	fclose(fp);
}



