#include <iostream.h>
#include "scene.h"
#include "spheres.h"
#include "polygon.h"
#include "ttransform.h"
#include "nff.h"
#include "strcom.h"

#define TIME 10.0

void add_rectX(Vector x,Float y,Float z,Surface_Info si)
{
	Vector V[4];
	V[0]=x; V[1]=x+Vector(0,y,0);
	V[2]=x+Vector(0,y,z); V[3]=x+Vector(0,0,z);
	K_add_object(Create_Polygon(4,V,0,TIME,si));
}
void add_rectY(Vector p,Float x,Float z,Surface_Info si)
{
	Vector V[4];
	V[0]=p; V[1]=p+Vector(x,0,0);
	V[2]=p+Vector(x,0,z); V[3]=p+Vector(0,0,z);
	K_add_object(Create_Polygon(4,V,0,TIME,si));
}
void add_rectZ(Vector p,Float x,Float y,Surface_Info si)
{
	Vector V[4];
	V[0]=p; V[1]=p+Vector(0,y,0);
	V[2]=p+Vector(x,y,0); V[3]=p+Vector(x,0,0);
	K_add_object(Create_Polygon(4,V,0,TIME,si));
}
void add_box(Vector x, Vector w, Surface_Info si)
{
	add_rectX(x,w.y,w.z,si);
	add_rectX(x+Vector(w.x,0,0),w.y,w.z,si);
	add_rectY(x,w.x,w.z,si);
	add_rectY(x+Vector(0,w.y,0),w.x,w.z,si);
	add_rectZ(x,w.x,w.y,si);
	add_rectZ(x+Vector(0,0,w.z),w.x,w.y,si);
}
void add_light_box(Vector x, Vector w, Surface_Info si)
{
	add_rectX(x,w.y,w.z,si);
	add_rectX(x+Vector(w.x,0,0),w.y,w.z,si);
	add_rectY(x,w.x,w.z,si);
	add_rectY(x+Vector(0,w.y,0),w.x,w.z,si);
	add_rectZ(x+Vector(0,0,w.z),w.x,w.y,si);
}
void add_building1(Vector p,Vector w,int stories, Float winheight,
Surface_Info brick, Surface_Info glass)
{
	add_box(p,w,brick);
	Float bot=p.z+w.z*0.2;
	Float height=(w.z)*0.8;
	Float floor_spacing=height/double(stories);
	for(int i=0;i<stories;i++){
		add_rectX(Vector(p.x-0.5, p.y+w.y*0.1, bot+i*floor_spacing),w.y*0.8,winheight*floor_spacing,glass);
		add_rectX(Vector(p.x+w.x+0.5, p.y+w.y*0.1, bot+i*floor_spacing),w.y*0.8,winheight*floor_spacing,glass);
		add_rectY(Vector(p.x+w.x*0.1, p.y-0.5, bot+i*floor_spacing),w.x*0.8,winheight*floor_spacing,glass);
		add_rectY(Vector(p.x+w.x*0.1, p.y+w.y+0.5, bot+i*floor_spacing),w.x*0.8,winheight*floor_spacing,glass);
	}
}

void do_the_animation()
{
Surface_Info si;
si.kdr=1; si.ksr=0; si.n=3;
si.dcolour=Colour(1,1,1);
si.rcolour=Colour(1,1,1);
// main ball
Object* sphere=Create_Static_Sphere(Vector(0,0,0),4,0,10,si);
Function** fns=new Function*[3];
fns[0]=Create_FN_ease_through(10,-40, 0,3);
fns[1]=new FN_constant(-40, 3,4);
fns[2]=Create_FN_ease_through(-40,-190, 4,10);
TTransformation*T=new TTransformation(0,10);
T->translate(TVector(
  new FN_composite(3,fns,0,10),
  new FN_constant(-10,0,10),
  new FN_constant(4,0,10)
  ));
K_add_object(Transform_Object(sphere,T));
// other balls
fns=new Function*[2];
fns[0]=Create_FN_linear_through(-40,40, 0,4);
fns[1]=new FN_constant(40, 4,10);
T=new TTransformation(0,10);
T->translate(TVector(
  new FN_constant(-75,0,10),
  new FN_composite(2,fns,0,10),
  new FN_constant(4,0,10)
  ));
K_add_object(Transform_Object(sphere,T));

fns=new Function*[2];
fns[0]=Create_FN_linear_through(40,-50, 0,4);
fns[1]=new FN_constant(-50, 4,10);
T=new TTransformation(0,10);
T->translate(TVector(
  new FN_constant(-55,0,10),
  new FN_composite(2,fns,0,10),
  new FN_constant(4,0,10)
  ));
K_add_object(Transform_Object(sphere,T));

fns=new Function*[3];
fns[0]=new FN_constant(-40, 0,5);
fns[1]=Create_FN_linear_through(-40,40, 5,7);
fns[2]=new FN_constant(40, 7,10);
T=new TTransformation(0,10);
T->translate(TVector(
  new FN_constant(-165,0,10),
  new FN_composite(3,fns,0,10),
  new FN_constant(4,0,10)
  ));
K_add_object(Transform_Object(sphere,T));

fns=new Function*[3];
fns[0]=new FN_constant(40, 0,7);
fns[1]=Create_FN_ease_through(40,25, 7,7.5);
fns[2]=new FN_constant(25, 7.5,10);
T=new TTransformation(0,10);
T->translate(TVector(
  new FN_constant(-145,0,10),
  new FN_composite(3,fns,0,10),
  new FN_constant(4,0,10)
  ));
K_add_object(Transform_Object(sphere,T));

// traffic lights
Vector V[4]={
Vector(-1,15,0),Vector(1,15,0),Vector(1,-15,0),Vector(-1,-15,0)};
si.dcolour=Colour(1,1,1);
si.kdr=1; si.ksr=0; si.n=1;
Object* poly=Create_Polygon(4,V,0,TIME,si);
T=new TTransformation(0,10);
fns=new Function*[3];
fns[0]=new FN_constant(0,0,3);
fns[1]=Create_FN_ease_through(0,PI/2,3,4);
fns[2]=new FN_constant(PI/2,4,10);
T->rotateZ(new FN_composite(3,fns,0,10));
T->translate(Create_TVector(-65,0,0,0,10));
K_add_object(Transform_Object(poly,T));

T=new TTransformation(0,10);
fns=new Function*[3];
fns[0]=new FN_constant(0,0,5);
fns[1]=Create_FN_ease_through(0,PI/2,5,7);
fns[2]=new FN_constant(PI/2,7,10);
T->rotateZ(new FN_composite(3,fns,0,10));
T->translate(Create_TVector(-155,0,0,0,10));
K_add_object(Transform_Object(poly,T));
}

void user_create_scene(Kernel_Info& ki,Camera& cam)
{

STRC.smart_time_divide=true;

cam.fname="test/output";
cam.width=100; cam.height=100;
Function **fns=new Function*[2];
fns[0]=Create_FN_ease_through(100,5 ,0,6);
fns[1]=new FN_constant(5,6,10);
Function *Zfn=new FN_composite(2,fns,0,10);
cam.C=
  TVector(
    Create_FN_ease_through(60,-115, 0,TIME),
    new FN_constant(0, 0,TIME),
    Zfn
  );
  //Create_TVector(60,0,100, 0,TIME);
cam.look_at=
  Create_TVector_ease_through(Vector(-60,0,5),Vector(-200,0,5),0,TIME);
  //Create_TVector(-60,0,0, 0,TIME);
cam.up=Create_TVector(0,0,1, 0,TIME);
cam.f=new FN_constant( 0.1,0,TIME);
cam.xs=cam.ys=new FN_constant(0.125, 0,TIME);
cam.end_time=TIME; cam.framesec=5;
cam.backdrop=Colour(0.2,0.2,0.2);
cam.sub=1;
cam.motion_blur=true;
cam.anti_alias=false;
ki.bounding_colour=cam.backdrop;
ki.ambient=Colour(0.4,0.4,0.4);
ki.seg_sub=2;
Surface_Info si,si2;

// buildings
si.kdr=0.9; si.ksr=0.0; si.n=1;
si.dcolour=Colour(0.4,0.8,0.9);
si.rcolour=Colour(1,1,1);
si2.kdr=0.6; si2.ksr=0.5; si2.n=6;
si2.dcolour=Colour(1,1,1); si2.rcolour=Colour(1,1,1);
add_building1(Vector(-10,-30,0),Vector(10,10,40),8,0.6,si,si2);

si.dcolour=Colour(0.8,0.6,0.6);
add_building1(Vector(-30,20,0),Vector(15,10,60),9,0.8,si,si2);

si.dcolour=Colour(0.8,0.9,0.9);
add_building1(Vector(-45,-30,0),Vector(30,10,70),12,0.5,si,si2);

si.dcolour=Colour(0.8,0.9,0.9);
add_building1(Vector(-45,20,0),Vector(10,30,35),5,0.8,si,si2);

si.dcolour=Colour(0.4,0.5,0.9);
add_building1(Vector(-95,-40,0),Vector(10,20,65),10,0.8,si,si2);

si.dcolour=Colour(0.4,0.9,0.5);
add_building1(Vector(-100,20,0),Vector(15,15,50),8,0.5,si,si2);

si.dcolour=Colour(0.9,0.6,0.4);
add_building1(Vector(-135,-35,0),Vector(35,15,40),8,0.5,si,si2);

si.dcolour=Colour(0.5,0.7,0.7);
add_building1(Vector(-135,20,0),Vector(30,10,55),10,0.6,si,si2);

si.dcolour=Colour(0.8,0.7,0.7);
add_building1(Vector(-195,-40,0),Vector(20,80,70),8,0.6,si,si2);


// road lines
si.dcolour=Colour(1,1,1);
si.kdr=1; si.ksr=0; si.n=1;
add_rectZ(Vector(-45,-1,0),60,2,si);
add_rectZ(Vector(-66,-60,0),2,40,si);
add_rectZ(Vector(-66,20,0),2,40,si);
add_rectZ(Vector(-135,-1,0),50,2,si);
add_rectZ(Vector(-156,-60,0),2,40,si);
add_rectZ(Vector(-156,20,0),2,40,si);

// floor polygon
si.kdr=0.4; si.ksr=0.5; si.n=4;
si.dcolour=Colour(0.5,0.5,0.5);
si.rcolour=Colour(1,1,1);
add_rectZ(Vector(-190,-50,-5),220,100,si);

// foot paths
si.kdr=0.7; si.ksr=0.0; si.n=1.5;
si.dcolour=Colour(0.7,0.7,0.7);
si.rcolour=Colour(1,1,1);
add_rectZ(Vector(-50,-20,0),55,5,si);
add_rectZ(Vector(-50,15,0),55,5,si);
add_rectZ(Vector(-140,-20,0),60,5,si);
add_rectZ(Vector(-140,15,0),60,5,si);
add_rectZ(Vector(-50,-60,0),5,40,si);
add_rectZ(Vector(-50,20,0),5,40,si);
add_rectZ(Vector(-85,-60,0),5,40,si);
add_rectZ(Vector(-85,20,0),5,40,si);
add_rectZ(Vector(-140,-60,0),5,40,si);
add_rectZ(Vector(-140,20,0),5,40,si);
add_rectZ(Vector(-175,-60,0),5,120,si);

// street lights
si.kdr=1; si.ksr=0; si.n=4;
si.dcolour=Colour(1,0.05,0.05);
K_add_light(new Light(
  Create_TVector(-65,0,50,0,TIME),
  Colour(0.7,0.7,0.7) ) );
add_light_box(Vector(-70,-5,48),Vector(10,10,3),si);

K_add_light(new Light(
  Create_TVector(-155,0,50,0,TIME),
  Colour(0.7,0.7,0.7) ) );
add_light_box(Vector(-160,-5,48),Vector(10,10,3),si);

do_the_animation();



}
