#ifndef __VECTOR_H
#define __VECTOR_H

#include "config.h"

class Vector{
public:
	Float x,y,z;
	Vector(Float X=0.0,Float Y=0.0,Float Z=0.0):
		x(X),y(Y),z(Z){}
	inline void normalize();
};

// operators

#include <math.h>

inline void Vector::normalize(){
	Float l=sqrt(x*x+y*y+z*z);
	if(l==0.0) return;
	l=1/l;
	x*=l; y*=l; z*=l;
}

inline Float length(const Vector& v)
{
	return sqrt( v.x*v.x + v.y*v.y + v.z*v.z );
}

inline Vector operator+(const Vector& a,const Vector& b)
{
	return Vector(a.x+b.x , a.y+b.y , a.z+b.z);
}


inline Vector operator-(const Vector& a,const Vector& b)
{
	return Vector(a.x-b.x , a.y-b.y , a.z-b.z);
}

inline Float operator*(const Vector& a,const Vector& b) // dot product
{
	return a.x*b.x + a.y*b.y + a.z*b.z;
}

inline Vector operator*(const Vector& a, Float s) // scale
{
	return Vector(a.x*s,a.y*s,a.z*s);
}
inline Vector operator*(Float s, const Vector& a)
{ return operator*(a,s);}

inline Vector cross(const Vector& a, const Vector& b) // cross product
{
	return Vector(a.y*b.z - a.z*b.y, a.z*b.x - a.x*b.z, a.x*b.y - a.y*b.x);
}

inline Vector operator-(const Vector& a)
{
	return Vector(-a.x,-a.y,-a.z);
}

#endif

